@@ -47,7 +47,7 @@ nepim_global_t nepim_global = {
   -1,      /* ttl */
   -1,      /* mcast_ttl */
   500000,  /* udp_greet_delay (usec) */
-  3,       /* max_greetings */
+  5,       /* max_greetings */
   2000000, /* udp_keepalive_timer (usec, receivers) */
   500000,  /* udp_keepalive_delay (usec, non-senders) */
   1,       /* udp_keepalive_require (boolean, receivers) */
@@ -56,6 +56,11 @@ nepim_global_t nepim_global = {
   -1,      /* win_recv */
   -1,      /* win_send */
   1,       /* udp_require_greet_reply (boolean, client) */
+  1,       /* seed */
+  0,       /* verify_data */
+  1,       /* random_fill */
+  0x00,    /* fill_byte */
+  0,       /* pause_duration */
   0,       /* password */
   0,       /* udp_exp_stats */
   0,       /* udp_exp_loss */
