@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: greet.c,v 1.9 2005/11/04 09:56:14 evertonm Exp $ */
+/* $Id: greet.c,v 1.10 2005/12/20 09:36:09 evertonm Exp $ */
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -37,6 +37,11 @@ const char *GREET_TEST_DURATION     = "test_duration=";
 const char *GREET_WRITE_DELAY       = "write_delay=";
 const char *GREET_SERVER_KA_SEND    = "server_ka_send=";
 const char *GREET_SERVER_KA_REQUIRE = "server_ka_req=";
+const char *GREET_SEED              = "seed=";
+const char *GREET_VERIFY_DATA       = "verify_data=";
+const char *GREET_RANDOM_FILL       = "random_fill=";
+const char *GREET_FILL_BYTE         = "fill_byte=";
+const char *GREET_PAUSE_DURATION    = "pause_duration=";
 const char *GREET_PASSWORD          = "password=";
 
 #define NEPIM_GREET_PARSE_OK     (0)
@@ -58,18 +63,23 @@ int nepim_write_greetings(const nepim_greet_t *opt, char *buf, int buf_size)
   assert(strlen(opt->password_buf) < opt->password_buf_size);
 
   pr = snprintf(buf, buf_size,
-		"hello %s%d %s%lld "
-		"%s%d %s%d %s%d "
-		"%s%ld %s%d %s%d %s%s\n", 
-		GREET_SERVER_SEND,       opt->must_send,
-		GREET_BIT_RATE,          opt->bit_rate,
-		GREET_PKT_RATE,          opt->pkt_rate,
-		GREET_STAT_INTERVAL,     opt->stat_interval,
-		GREET_TEST_DURATION,     opt->test_duration,
-		GREET_WRITE_DELAY,       opt->write_delay,
-		GREET_SERVER_KA_SEND,    opt->udp_keepalive_send,
-		GREET_SERVER_KA_REQUIRE, opt->udp_keepalive_require,
-		GREET_PASSWORD,          opt->password_buf);
+                "hello %s%d %s%lld "
+                "%s%d %s%d %s%d "
+                "%s%ld %s%d %s%d %s0x%08x %s%d %s%d %s0x%02x %s%d %s%s\n", 
+                GREET_SERVER_SEND,       opt->must_send,
+                GREET_BIT_RATE,          opt->bit_rate,
+                GREET_PKT_RATE,          opt->pkt_rate,
+                GREET_STAT_INTERVAL,     opt->stat_interval,
+                GREET_TEST_DURATION,     opt->test_duration,
+                GREET_WRITE_DELAY,       opt->write_delay,
+                GREET_SERVER_KA_SEND,    opt->udp_keepalive_send,
+                GREET_SERVER_KA_REQUIRE, opt->udp_keepalive_require,
+                GREET_SEED,              opt->seed,
+                GREET_VERIFY_DATA,       opt->verify_data,
+                GREET_RANDOM_FILL,       opt->random_fill,
+                GREET_FILL_BYTE,         opt->fill_byte,
+                GREET_PAUSE_DURATION,    opt->pause_duration,
+                GREET_PASSWORD,          opt->password_buf);
 
   if (pr < 1)
     return NEPIM_GREET_WRITE_IO;
@@ -83,7 +93,7 @@ int nepim_write_greetings(const nepim_greet_t *opt, char *buf, int buf_size)
 }
 
 int nepim_parse_greetings(nepim_greet_t *opt, 
-			  const char *buf, const char *past_end)
+                          const char *buf, const char *past_end)
 {
   char tmp[past_end - buf + 1];
   const char *SEP = " ";
@@ -98,6 +108,11 @@ int nepim_parse_greetings(nepim_greet_t *opt,
   const int GREET_WRITE_DELAY_LEN       = strlen(GREET_WRITE_DELAY);
   const int GREET_SERVER_KA_SEND_LEN    = strlen(GREET_SERVER_KA_SEND);
   const int GREET_SERVER_KA_REQUIRE_LEN = strlen(GREET_SERVER_KA_REQUIRE);
+  const int GREET_SEED_LEN              = strlen(GREET_SEED);
+  const int GREET_VERIFY_DATA_LEN       = strlen(GREET_VERIFY_DATA);
+  const int GREET_RANDOM_FILL_LEN       = strlen(GREET_RANDOM_FILL);
+  const int GREET_FILL_BYTE_LEN         = strlen(GREET_FILL_BYTE);
+  const int GREET_PAUSE_DURATION_LEN    = strlen(GREET_PAUSE_DURATION);
   const int GREET_PASSWORD_LEN          = strlen(GREET_PASSWORD);
 
   int server_send       = -2;
@@ -108,6 +123,11 @@ int nepim_parse_greetings(nepim_greet_t *opt,
   long write_delay      = -2;
   int server_ka_send    = -2;
   int server_ka_require = -2;
+  unsigned seed         = -2;
+  int verify_data       = -2;
+  int random_fill       = -2;
+  unsigned fill_byte    = -2;
+  int pause_duration    = -2;
 
   assert(opt->password_buf);
   assert(opt->password_buf_size > 0);
@@ -168,6 +188,40 @@ int nepim_parse_greetings(nepim_greet_t *opt,
       continue;
     }
 
+    if (!strncmp(tok, GREET_SEED, GREET_SEED_LEN)) {
+      if (sscanf(tok + GREET_SEED_LEN, "%x", &seed) != 1) {
+        fprintf(stderr, "%s: illegal seed value\n",
+                nepim_global.prog_name);
+        exit(1);
+      }
+      continue;
+    }
+
+    if (!strncmp(tok, GREET_VERIFY_DATA, GREET_VERIFY_DATA_LEN)) {
+      verify_data = atoi(tok + GREET_VERIFY_DATA_LEN);
+      continue;
+    }
+
+    if (!strncmp(tok, GREET_RANDOM_FILL, GREET_RANDOM_FILL_LEN)) {
+      random_fill = atoi(tok + GREET_RANDOM_FILL_LEN);
+      continue;
+    }
+
+    if (!strncmp(tok, GREET_FILL_BYTE, GREET_FILL_BYTE_LEN)) {
+      if ((sscanf(tok + GREET_FILL_BYTE_LEN, "%x", &fill_byte) != 1) ||
+          (fill_byte > 0xff)) {
+        fprintf(stderr, "%s: illegal fill byte value\n",
+                nepim_global.prog_name);
+        exit(1);
+      }
+      continue;
+    }
+
+    if (!strncmp(tok, GREET_PAUSE_DURATION, GREET_PAUSE_DURATION_LEN)) {
+      pause_duration = atoi(tok + GREET_PAUSE_DURATION_LEN);
+      continue;
+    }
+
     if (!strncmp(tok, GREET_PASSWORD, GREET_PASSWORD_LEN)) {
       const char *tmp_past_end = tmp + sizeof(tmp);
       const char *pass;
@@ -179,9 +233,9 @@ int nepim_parse_greetings(nepim_greet_t *opt,
       rem = tmp_past_end - pass;
       end = memchr(pass, ' ', rem);
       if (!end)
-	end = memchr(pass, '\0', rem);
+        end = memchr(pass, '\0', rem);
       if (!end)
-	end = tmp_past_end;
+        end = tmp_past_end;
       
       pass_len = NEPIM_MIN(opt->password_buf_size - 1, end - pass);
 
@@ -200,7 +254,7 @@ int nepim_parse_greetings(nepim_greet_t *opt,
     }
 
     fprintf(stderr, "%s %s: unknown greeting parameter: '%s'\n",
-	    __FILE__, __PRETTY_FUNCTION__, tok);
+            __FILE__, __PRETTY_FUNCTION__, tok);
   }
 
   assert(memchr(opt->password_buf, '\0', opt->password_buf_size));
@@ -218,6 +272,11 @@ int nepim_parse_greetings(nepim_greet_t *opt,
   opt->write_delay           = write_delay;
   opt->udp_keepalive_send    = server_ka_send;
   opt->udp_keepalive_require = server_ka_require;
+  opt->seed                  = seed;
+  opt->verify_data           = verify_data;
+  opt->random_fill           = random_fill;
+  opt->fill_byte             = (unsigned char) fill_byte;
+  opt->pause_duration        = pause_duration;
 
   return NEPIM_GREET_PARSE_OK;
 }
