@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: server.c,v 1.45 2005/11/04 09:56:14 evertonm Exp $ */
+/* $Id: server.c,v 1.46 2005/12/20 09:36:09 evertonm Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -54,7 +54,7 @@ static void tcp_pipe_kill(int sd);
 static void *on_tcp_rate_delay(oop_source *src, struct timeval tv, void *user);
 
 static void *on_tcp_read(oop_source *src, int sd,
-			 oop_event event, void *user)
+                         oop_event event, void *user)
 {
   char buf[nepim_global.tcp_read_size];
   int rd;
@@ -70,14 +70,14 @@ static void *on_tcp_read(oop_source *src, int sd,
     if (rd) {
       switch (errno) {
       case EINTR:
-	fprintf(stderr, "read: EINTR on TCP socket %d\n", sd);
-	return OOP_CONTINUE;
+        fprintf(stderr, "read: EINTR on TCP socket %d\n", sd);
+        return OOP_CONTINUE;
       case EAGAIN:
-	fprintf(stderr, "read: EAGAIN on TCP socket %d\n", sd);
-	return OOP_CONTINUE;
+        fprintf(stderr, "read: EAGAIN on TCP socket %d\n", sd);
+        return OOP_CONTINUE;
       default:
-	fprintf(stderr, "read: errno=%d: %s\n",
-		errno, strerror(errno));
+        fprintf(stderr, "read: errno=%d: %s\n",
+                errno, strerror(errno));
       }
     }
 
@@ -93,6 +93,9 @@ static void *on_tcp_read(oop_source *src, int sd,
 
   assert(rd >= 0);
   assert(rd <= sizeof(buf));
+  
+  /* modifies 'session->seed' */
+  tcp_check_data(sd, session, buf, rd);
 
   nepim_session_read_add(session, rd);
 
@@ -100,12 +103,13 @@ static void *on_tcp_read(oop_source *src, int sd,
 }
 
 static void *on_tcp_write(oop_source *src, int sd,
-			  oop_event event, void *user)
+                          oop_event event, void *user)
 {
   char buf[nepim_global.tcp_write_size];
   int wr;
   nepim_pipe_t *pipe = user;
   nepim_session_t *session = &pipe->session;
+  unsigned old_seed;
 
   assert(event == OOP_WRITE);
   assert(sd == pipe->sd);
@@ -114,8 +118,13 @@ static void *on_tcp_write(oop_source *src, int sd,
 
   assert(sizeof(buf) == nepim_global.tcp_write_size);
 
+  old_seed = session->seed;
+  /* modifies 'session->seed' */
+  fill_packet_data(session, buf, 0, nepim_global.tcp_write_size);
+
   wr = write(sd, buf, nepim_global.tcp_write_size);
   if (wr < 0) {
+    session->seed = old_seed;   /* rewind seed */
     switch (errno) {
     case EINTR:
       fprintf(stderr, "write: EINTR on TCP socket %d\n", sd);
@@ -137,6 +146,12 @@ static void *on_tcp_write(oop_source *src, int sd,
 
     return OOP_CONTINUE;
   }
+  if (wr < nepim_global.tcp_write_size) {
+      /* rewind seed to use exactly 'wr' bytes */
+      session->seed = old_seed;
+      /* modifies 'session->seed' */
+      fill_packet_data(session, buf, 0, wr);
+  }
 
   assert(wr >= 0);
   assert(wr <= sizeof(buf));
@@ -147,13 +162,14 @@ static void *on_tcp_write(oop_source *src, int sd,
 }
 
 static void *on_tcp_rate_write(oop_source *src, int sd,
-			       oop_event event, void *user)
+                               oop_event event, void *user)
 {
   char buf[nepim_global.tcp_write_size];
   int wr;
   nepim_pipe_t *pipe = user;
   nepim_session_t *session = &pipe->session;
   int to_write;
+  unsigned old_seed;
 
   assert(event == OOP_WRITE);
   assert(sd == pipe->sd);
@@ -189,8 +205,13 @@ static void *on_tcp_rate_write(oop_source *src, int sd,
   assert(to_write >= 0);
   assert(to_write <= sizeof(buf));
 
+  old_seed = session->seed;
+  /* modifies 'session->seed' */
+  fill_packet_data(session, buf, 0, to_write);
+
   wr = write(sd, buf, to_write);
   if (wr < 0) {
+    session->seed = old_seed;   /* rewind seed */
     switch (errno) {
     case EINTR:
       fprintf(stderr, "rate_write: EINTR on TCP socket %d\n", sd);
@@ -212,6 +233,12 @@ static void *on_tcp_rate_write(oop_source *src, int sd,
 
     return OOP_CONTINUE;
   }
+  if (wr < to_write) {
+      /* rewind seed to use exactly 'wr' bytes */
+      session->seed = old_seed;
+      /* modifies 'session->seed' */
+      fill_packet_data(session, buf, 0, wr);
+  }
 
   assert(wr >= 0);
   assert(wr <= sizeof(buf));
@@ -231,12 +258,12 @@ static void *on_tcp_rate_write(oop_source *src, int sd,
 
     /* stop writing */
     nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
-				    sd, OOP_WRITE);
+                                    sd, OOP_WRITE);
 
     /* schedule for next time saved by on_tcp_rate_delay() */
     nepim_global.oop_src->on_time(nepim_global.oop_src,
-				  session->tv_send_rate,
-				  on_tcp_rate_delay, pipe);
+                                  session->tv_send_rate,
+                                  on_tcp_rate_delay, pipe);
   }
 
   return OOP_CONTINUE;
@@ -268,9 +295,9 @@ static void *on_tcp_rate_delay(oop_source *src, struct timeval tv, void *user)
 
   /* calculate bytes to be written from rate */
   session->rate_remaining = nepim_bps2bytes(session->max_bit_rate,
-					    session->write_delay);
+                                            session->write_delay);
   session->pkt_remaining = nepim_pps2packets(session->max_pkt_rate,
-					     session->write_delay);
+                                             session->write_delay);
 
 #ifndef NDEBUG
  {
@@ -285,8 +312,8 @@ static void *on_tcp_rate_delay(oop_source *src, struct timeval tv, void *user)
 
   /* start to write */
   nepim_global.oop_src->on_fd(nepim_global.oop_src,
-			      pipe->sd, OOP_WRITE,
-			      on_tcp_rate_write, pipe);
+                              pipe->sd, OOP_WRITE,
+                              on_tcp_rate_write, pipe);
 
   return OOP_CONTINUE;
 }
@@ -297,13 +324,13 @@ static void *on_tcp_duration(oop_source *src, struct timeval tv, void *user)
   nepim_session_t *session = &pipe->session;
 
   nepim_pipe_stat(stdout, NEPIM_LABEL_TOTAL, pipe->sd, 
-		  session->byte_total_recv,
-		  session->byte_total_sent,
-		  session->test_duration,
-		  session->tv_start.tv_sec,
-		  session->test_duration,
-		  session->total_reads,
-		  session->total_writes);
+                  session->byte_total_recv,
+                  session->byte_total_sent,
+                  session->test_duration,
+                  session->tv_start.tv_sec,
+                  session->test_duration,
+                  session->total_reads,
+                  session->total_writes);
 
   session->byte_total_recv = 0;
   session->byte_total_sent = 0;
@@ -322,13 +349,13 @@ static void *on_tcp_interval(oop_source *src, struct timeval tv, void *user)
   nepim_session_t *session = &pipe->session;
 
   nepim_pipe_stat(stdout, NEPIM_LABEL_PARTIAL, pipe->sd, 
-		  session->byte_interval_recv,
-		  session->byte_interval_sent,
-		  session->stat_interval,
-		  session->tv_start.tv_sec,
-		  session->test_duration,
-		  session->interval_reads,
-		  session->interval_writes);
+                  session->byte_interval_recv,
+                  session->byte_interval_sent,
+                  session->stat_interval,
+                  session->tv_start.tv_sec,
+                  session->test_duration,
+                  session->interval_reads,
+                  session->interval_writes);
 
   session->byte_interval_recv = 0;
   session->byte_interval_sent = 0;
@@ -341,9 +368,9 @@ static void *on_tcp_interval(oop_source *src, struct timeval tv, void *user)
 }
 
 static int parse_greetings(int sd,
-			   const struct sockaddr *remote,
-			   socklen_t remote_len,
-			   const char *buf, const char *past_end)
+                           const struct sockaddr *remote,
+                           socklen_t remote_len,
+                           const char *buf, const char *past_end)
 {
   nepim_greet_t opt;
   int result;
@@ -357,19 +384,21 @@ static int parse_greetings(int sd,
     return result;
 
   fprintf(stderr, 
-	  "%d: send=%d bit_rate=%lld pkt_rate=%d "
-	  "interval=%d duration=%d delay=%ld ka_send=%d ka_req=%d "
-	  "password=%s\n", 
-	  sd, opt.must_send, opt.bit_rate, opt.pkt_rate,
-	  opt.stat_interval, opt.test_duration, 
-	  opt.write_delay, opt.udp_keepalive_send,
-	  opt.udp_keepalive_require,
-	  password_buf);
+          "%d: send=%d bit_rate=%lld pkt_rate=%d "
+          "interval=%d duration=%d delay=%ld ka_send=%d ka_req=%d seed=0x%08x "
+          "verify_data=%d random_fill=%d fill_byte=0x%02x pause_duration=%d "
+          "password=%s\n", 
+          sd, opt.must_send, opt.bit_rate, opt.pkt_rate,
+          opt.stat_interval, opt.test_duration, 
+          opt.write_delay, opt.udp_keepalive_send,
+          opt.udp_keepalive_require, opt.seed,
+          opt.verify_data, opt.random_fill, opt.fill_byte, opt.pause_duration,
+          password_buf);
 
   if (nepim_global.password)
     if (strcmp(nepim_global.password, password_buf)) {
       fprintf(stderr, "%d: bad client password=%s\n",
-	      sd, password_buf);
+              sd, password_buf);
       return 1;
     }
 
@@ -379,8 +408,8 @@ static int parse_greetings(int sd,
 }
 
 static int read_greetings(int sd,
-			  const struct sockaddr *remote,
-			  socklen_t remote_len)
+                          const struct sockaddr *remote,
+                          socklen_t remote_len)
 {
   char buf[1024];
   int len = 0;
@@ -389,25 +418,22 @@ static int read_greetings(int sd,
 
   for (;;) {
     char *curr = buf + len;
-    int left = sizeof(buf) - len;
     int rd;
 
-    assert(left > 0);
-
-    rd = read(sd, curr, left);
+    /* Beware to not eat part of the next packet. */
+    rd = read(sd, curr, 1);
     if (!rd) {
       fprintf(stderr, 
-	      "%s: incoming connection lost\n", 
-	      __PRETTY_FUNCTION__);
+              "%s: incoming connection lost\n", 
+              __PRETTY_FUNCTION__);
       return -1;
     }
     if (rd < 0) {
       if (errno == EAGAIN)
-	continue;
+        continue;
     }
 
     assert(rd > 0);
-    assert(rd <= left);
 
     len += rd;
 
@@ -421,8 +447,8 @@ static int read_greetings(int sd,
   result = parse_greetings(sd, remote, remote_len, buf, eos);
   if (result) {
     fprintf(stderr, 
-	    "%d: bad client greetings: %d [%s]\n", 
-	    sd, result, buf);
+            "%d: bad client greetings: %d [%s]\n", 
+            sd, result, buf);
     return result;
   }
 
@@ -445,8 +471,8 @@ static void schedule_stat_interval(int sd)
   session->tv_interval.tv_sec += session->stat_interval;
 
   nepim_global.oop_src->on_time(nepim_global.oop_src,
-				session->tv_interval,
-				on_tcp_interval, pipe);
+                                session->tv_interval,
+                                on_tcp_interval, pipe);
 }
 
 static void tcp_pipe_start(int sd)
@@ -458,20 +484,20 @@ static void tcp_pipe_start(int sd)
   session = &pipe->session;
 
   nepim_global.oop_src->on_fd(nepim_global.oop_src,
-			      sd, OOP_READ,
-			      on_tcp_read, pipe);
+                              sd, OOP_READ,
+                              on_tcp_read, pipe);
 
   if (session->must_send) {
     if ((session->max_bit_rate > 0) || (session->max_pkt_rate > 0)) {
       session->tv_send_rate = OOP_TIME_NOW;
       nepim_global.oop_src->on_time(nepim_global.oop_src,
-				    session->tv_send_rate,
-				    on_tcp_rate_delay, pipe);
+                                    session->tv_send_rate,
+                                    on_tcp_rate_delay, pipe);
     }
     else {
       nepim_global.oop_src->on_fd(nepim_global.oop_src,
-				  sd, OOP_WRITE,
-				  on_tcp_write, pipe);
+                                  sd, OOP_WRITE,
+                                  on_tcp_write, pipe);
     }
   }
 
@@ -482,14 +508,40 @@ static void tcp_pipe_start(int sd)
   }
   session->tv_duration.tv_sec += session->test_duration;
   nepim_global.oop_src->on_time(nepim_global.oop_src,
-				session->tv_duration,
-				on_tcp_duration, pipe);
+                                session->tv_duration,
+                                on_tcp_duration, pipe);
 
   nepim_sock_show_opt(stderr, sd);
 
   schedule_stat_interval(sd);
 }
 
+static void *on_tcp_pipe_start(oop_source *src, struct timeval tv, void *user)
+{
+  nepim_pipe_t *pipe = user;
+
+  tcp_pipe_start(pipe->sd);
+  return OOP_CONTINUE;
+}
+
+static void schedule_tcp_pipe_start(int sd)
+{
+    nepim_pipe_t *pipe = nepim_pipe_set_get(&pipes, sd);
+    nepim_session_t *session;
+    struct timeval pause;
+
+    session = &pipe->session;
+
+    {
+        int result = gettimeofday(&pause, 0);
+        assert(!result);
+    }
+    pause.tv_sec += session->pause_duration;
+    nepim_global.oop_src->on_time(nepim_global.oop_src,
+                                  pause,
+                                  on_tcp_pipe_start, pipe);
+}
+
 static void tcp_pipe_cancel_timers(int sd)
 {
   nepim_pipe_t *pipe = nepim_pipe_set_get(&pipes, sd);
@@ -499,12 +551,12 @@ static void tcp_pipe_cancel_timers(int sd)
   session = &pipe->session;
 
   nepim_global.oop_src->cancel_time(nepim_global.oop_src,
-				    session->tv_duration,
-				    on_tcp_duration, pipe);
+                                    session->tv_duration,
+                                    on_tcp_duration, pipe);
 
   nepim_global.oop_src->cancel_time(nepim_global.oop_src,
-				    session->tv_interval,
-				    on_tcp_interval, pipe);
+                                    session->tv_interval,
+                                    on_tcp_interval, pipe);
 }
 
 static void tcp_pipe_cancel_io(int sd)
@@ -516,22 +568,22 @@ static void tcp_pipe_cancel_io(int sd)
   session = &pipe->session;
 
   nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
-				  sd, OOP_READ);
+                                  sd, OOP_READ);
 
   if (session->must_send) {
     if ((session->max_bit_rate > 0) || (session->max_pkt_rate > 0)) {
       /* stop current writing, if any */
       nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
-				      sd, OOP_WRITE);
+                                      sd, OOP_WRITE);
       
       /* stop periodic write scheduler */
       nepim_global.oop_src->cancel_time(nepim_global.oop_src,
-					session->tv_send_rate,
-					on_tcp_rate_delay, pipe);
+                                        session->tv_send_rate,
+                                        on_tcp_rate_delay, pipe);
     }
     else {
       nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
-				      sd, OOP_WRITE);
+                                      sd, OOP_WRITE);
     }
   }
 }
@@ -553,7 +605,7 @@ static void tcp_pipe_kill(int sd)
 }
 
 static void *on_tcp_connect(oop_source *src, int sd,
-			    oop_event event, void *unnused)
+                            oop_event event, void *unnused)
 {
   int conn_sd;
   union {
@@ -567,38 +619,38 @@ static void *on_tcp_connect(oop_source *src, int sd,
   conn_sd = accept(sd, (struct sockaddr *) &sa, &len);
   if (conn_sd < 0) {
     fprintf(stderr, 
-	    "%s: %s: could not accept connection: errno=%d: %s\n", 
-	    __FILE__, __PRETTY_FUNCTION__, errno, strerror(errno));
+            "%s: %s: could not accept connection: errno=%d: %s\n", 
+            __FILE__, __PRETTY_FUNCTION__, errno, strerror(errno));
     return OOP_CONTINUE;
   }
 
   nepim_sock_dump_addr(addr_buf, sizeof(addr_buf),
-		       (const struct sockaddr *) &sa);
+                       (const struct sockaddr *) &sa);
 
   fprintf(stderr,
-	  "%d: TCP incoming: %s,%d\n",
-	  conn_sd, addr_buf, 
-	  nepim_sock_get_port((const struct sockaddr *) &sa));
+          "%d: TCP incoming: %s,%d\n",
+          conn_sd, addr_buf, 
+          nepim_sock_get_port((const struct sockaddr *) &sa));
 
   if (nepim_global.tcpwrap) {
     if (!nepim_hosts_ctl(nepim_global.tcpwrap,
-			 nepim_global.prog_name,
-			 addr_buf)) {
+                         nepim_global.prog_name,
+                         addr_buf)) {
       fprintf(stderr, 
-	      "%d: %s: %s: TCP wrapper denied access from client=[%s] to service=[%s]\n",
-	      conn_sd, __FILE__, __PRETTY_FUNCTION__,
-	      addr_buf, nepim_global.prog_name);
+              "%d: %s: %s: TCP wrapper denied access from client=[%s] to service=[%s]\n",
+              conn_sd, __FILE__, __PRETTY_FUNCTION__,
+              addr_buf, nepim_global.prog_name);
       close(conn_sd);
       return OOP_CONTINUE;
     }
   }
 
   result = nepim_socket_opt(conn_sd, nepim_global.pmtu_mode, 
-			    nepim_global.ttl);
+                            nepim_global.ttl);
   if (result) {
     fprintf(stderr, 
-	    "%d: %s: %s: could not set socket options: %d\n",
-	    conn_sd, __FILE__, __PRETTY_FUNCTION__, result);
+            "%d: %s: %s: could not set socket options: %d\n",
+            conn_sd, __FILE__, __PRETTY_FUNCTION__, result);
     close(conn_sd);
     return OOP_CONTINUE;
   }
@@ -606,37 +658,37 @@ static void *on_tcp_connect(oop_source *src, int sd,
   result = nepim_socket_tcp_opt(conn_sd);
   if (result) {
     fprintf(stderr, 
-	    "%d: %s: %s: could not set tcp options: %d\n",
-	    conn_sd, __FILE__, __PRETTY_FUNCTION__, result);
+            "%d: %s: %s: could not set tcp options: %d\n",
+            conn_sd, __FILE__, __PRETTY_FUNCTION__, result);
     close(conn_sd);
     return OOP_CONTINUE;
   }
 
   if (nepim_socket_block(conn_sd)) {
     fprintf(stderr, 
-	    "%d: %s: %s: could not set blocking socket mode\n",
-	    conn_sd, __FILE__, __PRETTY_FUNCTION__);
+            "%d: %s: %s: could not set blocking socket mode\n",
+            conn_sd, __FILE__, __PRETTY_FUNCTION__);
     close(conn_sd);
     return OOP_CONTINUE;
   }
 
   if (read_greetings(conn_sd, (const struct sockaddr *) &sa, len)) {
     fprintf(stderr, 
-	    "%d: %s: %s: could not parse client greetings\n",
-	    conn_sd, __FILE__, __PRETTY_FUNCTION__);
+            "%d: %s: %s: could not parse client greetings\n",
+            conn_sd, __FILE__, __PRETTY_FUNCTION__);
     close(conn_sd);
     return OOP_CONTINUE;
   }
 
   if (nepim_socket_nonblock(conn_sd)) {
     fprintf(stderr, 
-	    "%d: %s: %s: could not set non-blocking socket mode\n",
-	    conn_sd, __FILE__, __PRETTY_FUNCTION__);
+            "%d: %s: %s: could not set non-blocking socket mode\n",
+            conn_sd, __FILE__, __PRETTY_FUNCTION__);
     close(conn_sd);
     return OOP_CONTINUE;
   }
 
-  tcp_pipe_start(conn_sd);
+  schedule_tcp_pipe_start(conn_sd);
 
   return OOP_CONTINUE;
 }
@@ -662,7 +714,7 @@ static int spawn_tcp_listener(const char *hostname, const char *portname)
   result = getaddrinfo(hostname, portname, &hints, &ai_res);
   if (result) {
     fprintf(stderr, "%s: getaddrinfo(%s,%s): %s\n",
-	    __PRETTY_FUNCTION__,
+            __PRETTY_FUNCTION__,
             hostname, portname, gai_strerror(result));
     return 0;
   }
@@ -672,10 +724,10 @@ static int spawn_tcp_listener(const char *hostname, const char *portname)
 
 #if 0
     fprintf(stderr, 
-	    "TCP listener: host=%s,%s(%d) len=%d family=%d type=%d proto=%d\n",
-	    hostname, portname, nepim_sock_get_port(ai->ai_addr),
-	    ai->ai_addrlen, ai->ai_family, 
-	    ai->ai_socktype, ai->ai_protocol);
+            "TCP listener: host=%s,%s(%d) len=%d family=%d type=%d proto=%d\n",
+            hostname, portname, nepim_sock_get_port(ai->ai_addr),
+            ai->ai_addrlen, ai->ai_family, 
+            ai->ai_socktype, ai->ai_protocol);
 #endif
 
     if (nepim_global.no_inet6 && (ai->ai_family == PF_INET6))
@@ -685,30 +737,30 @@ static int spawn_tcp_listener(const char *hostname, const char *portname)
       continue;
 
     sd = nepim_create_listener_socket(ai->ai_addr, ai->ai_addrlen,
-				      ai->ai_family, ai->ai_socktype, 
-				      ai->ai_protocol,
-				      nepim_global.listen_backlog,
-				      nepim_global.pmtu_mode,
-				      nepim_global.ttl,
-				      nepim_global.win_recv,
-				      nepim_global.win_send);
+                                      ai->ai_family, ai->ai_socktype, 
+                                      ai->ai_protocol,
+                                      nepim_global.listen_backlog,
+                                      nepim_global.pmtu_mode,
+                                      nepim_global.ttl,
+                                      nepim_global.win_recv,
+                                      nepim_global.win_send);
     if (sd < 0) {
       fprintf(stderr, 
-	      "%s: TCP listener socket failed for %s,%s(%d): %d\n",
-	      __PRETTY_FUNCTION__, hostname, portname, 
-	      nepim_sock_get_port(ai->ai_addr), sd);
+              "%s: TCP listener socket failed for %s,%s(%d): %d\n",
+              __PRETTY_FUNCTION__, hostname, portname, 
+              nepim_sock_get_port(ai->ai_addr), sd);
       break;
     }
 
     nepim_global.oop_src->on_fd(nepim_global.oop_src,
-			  	sd, OOP_READ,
-				on_tcp_connect, 0);
+                                sd, OOP_READ,
+                                on_tcp_connect, 0);
 
     ++tcp_listeners;
 
     fprintf(stderr, 
-	    "%d: TCP socket listening on %s,%s(%d)\n",
-	    sd, hostname, portname, nepim_sock_get_port(ai->ai_addr));
+            "%d: TCP socket listening on %s,%s(%d)\n",
+            sd, hostname, portname, nepim_sock_get_port(ai->ai_addr));
   }
 
   freeaddrinfo(ai_res);
@@ -717,7 +769,7 @@ static int spawn_tcp_listener(const char *hostname, const char *portname)
 }
 
 extern int nepim_udp_listener(const char *hostname,
-			      const char *portname, int join);
+                              const char *portname, int join);
 
 static void parse_listeners(const char *list, int *udp, int *tcp, int join)
 {
@@ -731,8 +783,8 @@ static void parse_listeners(const char *list, int *udp, int *tcp, int join)
 
     if (addr_list_get(list, i, buf, sizeof(buf))) {
       fprintf(stderr, 
-	      "%s: failure parsing address %d/%d from list: %s\n",
-	      nepim_global.prog_name, i, size, list);
+              "%s: failure parsing address %d/%d from list: %s\n",
+              nepim_global.prog_name, i, size, list);
       continue;
     }
 
@@ -757,11 +809,11 @@ void nepim_server_run()
 
   if (nepim_global.bind_list)
     parse_listeners(nepim_global.bind_list, &udp_listeners, &tcp_listeners,
-		    0 /* no join */);
+                    0 /* no join */);
 
   if (nepim_global.join_list)
     parse_listeners(nepim_global.join_list, &udp_listeners, &tcp_listeners,
-		    1 /* do join */);
+                    1 /* do join */);
 
   if (!nepim_global.bind_list && !nepim_global.join_list) {
     if (!nepim_global.no_inet6)
@@ -769,9 +821,9 @@ void nepim_server_run()
     tcp_listeners += spawn_tcp_listener(INET_ANY, nepim_global.portname);
     if (!nepim_global.no_inet6)
       udp_listeners += nepim_udp_listener(INET6_ANY, nepim_global.portname,
-					  0 /* no join */);
+                                          0 /* no join */);
     udp_listeners += nepim_udp_listener(INET_ANY, nepim_global.portname,
-					0 /* no join */);
+                                        0 /* no join */);
   }
   
   if (tcp_listeners || udp_listeners)
