@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: session.c,v 1.7 2005/11/04 09:56:15 evertonm Exp $ */
+/* $Id: session.c,v 1.8 2005/12/20 09:36:09 evertonm Exp $ */
 
 #include <string.h>
 #include <assert.h>
@@ -26,8 +26,8 @@ MA 02110-1301 USA
 #include "session.h"
 
 void nepim_session_init(nepim_session_t *session, const nepim_greet_t *opt,
-			const struct sockaddr *remote, socklen_t remote_len,
-			nepim_session_type type, int index)
+                        const struct sockaddr *remote, socklen_t remote_len,
+                        nepim_session_type type, int index)
 {
   switch(type) {
   case SESSION_PIPE:
@@ -54,6 +54,12 @@ void nepim_session_init(nepim_session_t *session, const nepim_greet_t *opt,
   session->write_delay           = opt->write_delay;
   session->udp_keepalive_send    = opt->udp_keepalive_send;
   session->udp_keepalive_require = opt->udp_keepalive_require;
+  session->seed                  = opt->seed;
+  session->check_seed            = opt->seed;        /* see tcp_check_data() */
+  session->verify_data           = opt->verify_data;
+  session->random_fill           = opt->random_fill;
+  session->fill_byte             = opt->fill_byte;
+  session->pause_duration        = opt->pause_duration;
 
   session->duration_done      = 0;
   session->total_reads        = 0;
