@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: session.h,v 1.12 2005/11/04 09:56:15 evertonm Exp $ */
+/* $Id: session.h,v 1.13 2005/12/20 09:36:09 evertonm Exp $ */
 
 #ifndef NEPIM_SESSION_H
 #define NEPIM_SESSION_H
@@ -58,6 +58,12 @@ struct nepim_session_t {
   int duration_done;         /* boolean */
   int udp_keepalive_send;    /* boolean, non-senders */
   int udp_keepalive_require; /* boolean, receivers */
+  unsigned seed;
+  unsigned check_seed;	     /* used only by tcp_check_data() */
+  int verify_data;	     /* boolean, client */
+  int random_fill;	     /* boolean, client */
+  unsigned char fill_byte;
+  int pause_duration;
 
   int rate_remaining;        /* bytes, used when max_bit_rate > 0 */
   int pkt_remaining;         /* pkts, used when max_pkt_rate > 0 */
