@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: slot.c,v 1.28 2005/11/04 09:56:15 evertonm Exp $ */
+/* $Id: slot.c,v 1.29 2005/12/20 09:36:09 evertonm Exp $ */
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -69,8 +69,8 @@ static int match_remote_addr(const void *context, const void *element)
 }
 
 nepim_slot_t *nepim_slot_set_search_remote(const nepim_slot_set_t *set,
-					   const struct sockaddr *addr,
-					   socklen_t len)
+                                           const struct sockaddr *addr,
+                                           socklen_t len)
 {
   struct ctx_t ctx = { addr, len };
   return nepim_array_find(&set->array, match_remote_addr, &ctx);
@@ -82,10 +82,10 @@ int nepim_slot_find_free(nepim_slot_set_t *set)
 }
 
 void nepim_slot_set_add(nepim_slot_set_t *set, int sd,
-			int index, int index_remote,
-			const struct sockaddr *remote,
-			socklen_t remote_len,
-			const nepim_greet_t *opt)
+                        int index, int index_remote,
+                        const struct sockaddr *remote,
+                        socklen_t remote_len,
+                        const nepim_greet_t *opt)
 {
   nepim_slot_t *slot = malloc(sizeof(*slot));
   assert(slot);
@@ -107,8 +107,8 @@ void nepim_slot_set_add(nepim_slot_set_t *set, int sd,
   slot->exp_dup_sampler   = 0;
   
   nepim_session_init(&slot->session, opt, 
-		     remote, remote_len,
-		     SESSION_SLOT, index);
+                     remote, remote_len,
+                     SESSION_SLOT, index);
 
   nepim_win_init(&slot->win_recv_seq, nepim_global.udp_win_max);
   
@@ -135,8 +135,8 @@ static int slot_writer(nepim_slot_t *slot, int sd)
   if (slot)
     if (slot->udp_sd == sd)
       if (slot->want_write) {
-	assert(slot->want_write == 1);
-	return -1;
+        assert(slot->want_write == 1);
+        return -1;
       }
   
   return 0;
@@ -165,9 +165,9 @@ nepim_slot_t *nepim_slot_find_next_writer(const nepim_slot_set_t *set, int sd)
 }
 
 int nepim_slot_find_addr(const nepim_slot_set_t *set,
-			 int remote_slot, 
-			 const struct sockaddr *remote,
-			 socklen_t remote_len)
+                         int remote_slot, 
+                         const struct sockaddr *remote,
+                         socklen_t remote_len)
 {
   nepim_slot_t *slot;
   int i;
@@ -176,9 +176,9 @@ int nepim_slot_find_addr(const nepim_slot_set_t *set,
     slot = nepim_slot_set_get(set, i);
     if (slot)
       if (slot->index_remote == remote_slot)
-	if (slot->session.remote_len == remote_len)
-	  if (!memcmp(&slot->session.remote, remote, remote_len))
-	    return -1;
+        if (slot->session.remote_len == remote_len)
+          if (!memcmp(&slot->session.remote, remote, remote_len))
+            return -1;
   }
 
   return 0;
@@ -194,17 +194,21 @@ static void slot_buf_write(nepim_slot_t *slot, char *buf, size_t buf_size, uint8
   hdr.dst_slot = slot->index_remote;
   hdr.src_slot = slot->index;
   hdr.type     = type;
+  hdr.seed     = slot->session.seed;
 
   switch (type) {
   case UDP_TYPE_DATA:
 
+    /* modifies 'session->seed' */
+    fill_packet_data(&(slot->session), buf, UDP_HEADER_LEN, buf_size);
+
     if (nepim_global.udp_exp_loss > 0)
       if (!(++slot->exp_loss_sampler % nepim_global.udp_exp_loss))
-	++slot->seq;
+        ++slot->seq;
 
     if (nepim_global.udp_exp_dup > 0)
       if (!(++slot->exp_dup_sampler % nepim_global.udp_exp_dup))
-	--slot->seq;
+        --slot->seq;
 
     hdr.seq = slot->seq++;
 
@@ -227,8 +231,8 @@ int nepim_slot_buf_sendto(nepim_slot_t *slot, char *buf, size_t buf_size, uint8_
   slot_buf_write(slot, buf, buf_size, type);
 
   wr = sendto(slot->udp_sd, buf, buf_size, 0,
-	      (const struct sockaddr *) &slot->session.remote,
-	      slot->session.remote_len);
+              (const struct sockaddr *) &slot->session.remote,
+              slot->session.remote_len);
 
   return wr;
 }
@@ -259,7 +263,7 @@ void nepim_will_slot_keepalive(nepim_slot_t *slot, oop_call_fd *on_udp_write)
     return;
 
   nepim_global.oop_src->on_fd(nepim_global.oop_src, sd,
-			      OOP_WRITE, on_udp_write, 0);
+                              OOP_WRITE, on_udp_write, 0);
 }
 
 void nepim_cancel_slot_keepalive(nepim_slot_t *slot)
@@ -277,14 +281,14 @@ void nepim_cancel_slot_keepalive(nepim_slot_t *slot)
     return;
 
   nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
-				  sd, OOP_WRITE);
+                                  sd, OOP_WRITE);
 }
 
 void nepim_udp_write_keepalive(nepim_slot_t *slot, 
-			       int (*nepim_slot_buf_send)(nepim_slot_t*,
-							  char*,size_t,uint8_t),
-			       oop_call_time *on_udp_keepalive_time,
-			       void (*udp_slot_kill)(nepim_slot_t *))
+                               int (*nepim_slot_buf_send)(nepim_slot_t*,
+                                                          char*,size_t,uint8_t),
+                               oop_call_time *on_udp_keepalive_time,
+                               void (*udp_slot_kill)(nepim_slot_t *))
 {
   char buf[nepim_global.udp_write_size];
   int wr;
@@ -315,8 +319,8 @@ void nepim_udp_write_keepalive(nepim_slot_t *slot,
     case EINTR:
     case EAGAIN:
       nepim_usock_write_error(&udp_tab, sd, 
-			      slot->index, slot->index_remote,
-			      errno);
+                              slot->index, slot->index_remote,
+                              errno);
 
       return;
     case EPIPE:
@@ -325,8 +329,8 @@ void nepim_udp_write_keepalive(nepim_slot_t *slot,
     }
 
     fprintf(stderr,
-	    "%d: keepalive_write: unexpected failure: errno=%d: %s\n",
-	    sd, errno, strerror(errno));
+            "%d: keepalive_write: unexpected failure: errno=%d: %s\n",
+            sd, errno, strerror(errno));
 
     if (!session->duration_done)
       report_broken_slot_stat(stdout, slot);
@@ -348,12 +352,12 @@ void nepim_udp_write_keepalive(nepim_slot_t *slot,
 
   /* schedule for next time saved by on_udp_rate_delay() */
   nepim_global.oop_src->on_time(nepim_global.oop_src,
-				session->tv_keepalive,
-				on_udp_keepalive_time, slot);
+                                session->tv_keepalive,
+                                on_udp_keepalive_time, slot);
 }
 
 void nepim_schedule_keepalive_timer(nepim_slot_t *slot,
-				    oop_call_time *on_udp_keepalive_require)
+                                    oop_call_time *on_udp_keepalive_require)
 {
   nepim_session_t *session = &slot->session;
 
@@ -364,9 +368,9 @@ void nepim_schedule_keepalive_timer(nepim_slot_t *slot,
     assert(!result);
   }
   nepim_timer_usec_add(&session->tv_keepalive_timer, 
-		       nepim_global.udp_keepalive_timer);
+                       nepim_global.udp_keepalive_timer);
 
   nepim_global.oop_src->on_time(nepim_global.oop_src,
-				session->tv_keepalive_timer,
-				on_udp_keepalive_require, slot);
+                                session->tv_keepalive_timer,
+                                on_udp_keepalive_require, slot);
 }
