@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: udp_client.c,v 1.46 2005/11/04 09:56:15 evertonm Exp $ */
+/* $Id: udp_client.c,v 1.47 2005/12/20 09:36:09 evertonm Exp $ */
 
 
 #include <stdio.h>
@@ -44,13 +44,13 @@ extern nepim_usock_set_t udp_tab; /* from server.c */
 
 static void *on_udp_rate_delay(oop_source *src, struct timeval tv, void *user);
 static void *on_udp_write(oop_source *src, int sd,
-			  oop_event event, void *user);
+                          oop_event event, void *user);
 static void udp_slot_cancel_stat_timers(nepim_slot_t *slot);
 static void schedule_stat_interval(nepim_slot_t *slot);
 static void udp_slot_kill(nepim_slot_t *slot);
 static void *on_udp_duration(oop_source *src, struct timeval tv, void *user);
 static void *on_udp_time_greet(oop_source *src, struct timeval tv,
-			       void *user);
+                               void *user);
 static void soft_cancel_slot_greet_writer(nepim_slot_t *slot);
 static void cancel_slot_greet_writer(nepim_slot_t *slot);
 static void cancel_slot_greet_timer(nepim_slot_t *slot);
@@ -71,7 +71,7 @@ static void will_slot_write(nepim_slot_t *slot)
     return;
 
   nepim_global.oop_src->on_fd(nepim_global.oop_src, sd,
-			      OOP_WRITE, on_udp_write, 0);
+                              OOP_WRITE, on_udp_write, 0);
 }
 
 static void *on_udp_keepalive_require(oop_source *src, struct timeval tv, void *user)
@@ -84,8 +84,8 @@ static void *on_udp_keepalive_require(oop_source *src, struct timeval tv, void *
 
   if (slot->keepalives_recv < 1) {
     fprintf(stderr, 
-	    "%d %d-%d: broken slot: peer not sending keepalives?\n",
-	    slot->udp_sd, slot->index, slot->index_remote);
+            "%d %d-%d: broken slot: peer not sending keepalives?\n",
+            slot->udp_sd, slot->index, slot->index_remote);
 
     report_broken_slot_stat(stdout, slot);
 
@@ -133,8 +133,8 @@ static void udp_slot_start(nepim_slot_t *slot)
     if ((session->max_bit_rate > 0) || (session->max_pkt_rate > 0)) {
       session->tv_send_rate = OOP_TIME_NOW;
       nepim_global.oop_src->on_time(nepim_global.oop_src,
-				    session->tv_send_rate,
-				    on_udp_rate_delay, slot);
+                                    session->tv_send_rate,
+                                    on_udp_rate_delay, slot);
     }
     else {
       will_slot_write(slot);
@@ -144,14 +144,14 @@ static void udp_slot_start(nepim_slot_t *slot)
     if (session->udp_keepalive_send) {
 
       fprintf(stderr, 
-	      "DEBUG %s %s: %d %d-%d: scheduling sending of keepalives\n",
-	      __FILE__, __PRETTY_FUNCTION__,
-	      slot->udp_sd, slot->index, slot->index_remote);
+              "DEBUG %s %s: %d %d-%d: scheduling sending of keepalives\n",
+              __FILE__, __PRETTY_FUNCTION__,
+              slot->udp_sd, slot->index, slot->index_remote);
       
       session->tv_keepalive = OOP_TIME_NOW;
       nepim_global.oop_src->on_time(nepim_global.oop_src,
-				    session->tv_keepalive,
-				    on_udp_keepalive_time, slot);
+                                    session->tv_keepalive,
+                                    on_udp_keepalive_time, slot);
     }
   }
 
@@ -162,8 +162,8 @@ static void udp_slot_start(nepim_slot_t *slot)
   }
   session->tv_duration.tv_sec += session->test_duration;
   nepim_global.oop_src->on_time(nepim_global.oop_src,
-				session->tv_duration,
-				on_udp_duration, slot);
+                                session->tv_duration,
+                                on_udp_duration, slot);
 
   nepim_sock_show_opt(stderr, slot->udp_sd);
 
@@ -173,6 +173,32 @@ static void udp_slot_start(nepim_slot_t *slot)
     nepim_schedule_keepalive_timer(slot, on_udp_keepalive_require);
 }
 
+static void *on_udp_slot_start(oop_source *src, struct timeval tv, void *user)
+{
+  nepim_slot_t *slot = user;
+
+  udp_slot_start(slot);
+
+  return OOP_CONTINUE;
+}
+
+static void schedule_udp_slot_start(nepim_slot_t *slot)
+{
+  nepim_session_t *session;
+  struct timeval pause;
+
+  session = &slot->session;
+
+  {
+      int result = gettimeofday(&pause, 0);
+      assert(!result);
+  }
+  pause.tv_sec += session->pause_duration;
+  nepim_global.oop_src->on_time(nepim_global.oop_src,
+                                pause,
+                                on_udp_slot_start, slot);
+}
+
 static void udp_consume(nepim_slot_t *slot, size_t len, uint32_t seq)
 {
   nepim_session_t *session = &slot->session;
@@ -184,11 +210,11 @@ static void udp_consume(nepim_slot_t *slot, size_t len, uint32_t seq)
 
   if (nepim_global.udp_exp_stats)
     nepim_win_add(&slot->win_recv_seq, seq,
-		  &slot->interval_pkt_lost, &slot->interval_pkt_dup);
+                  &slot->interval_pkt_lost, &slot->interval_pkt_dup);
 }
 
 static void *on_udp_read_segment(oop_source *src, int sd,
-				 oop_event event, void *user)
+                                 oop_event event, void *user)
 {
   int rd;
   char buf[nepim_global.udp_read_size];
@@ -204,7 +230,7 @@ static void *on_udp_read_segment(oop_source *src, int sd,
   assert(!user);
 
   rd = recvfrom(sd, buf, nepim_global.udp_read_size, 0, 
-		(struct sockaddr *) &from, &fromlen);
+                (struct sockaddr *) &from, &fromlen);
   if (rd < 0) {
     switch (errno) {
     case EINTR:
@@ -215,8 +241,8 @@ static void *on_udp_read_segment(oop_source *src, int sd,
     }
 
     fprintf(stderr, 
-	    "%d: recvfrom: unexpected failure: errno=%d: %s\n",
-	    sd, errno, strerror(errno));
+            "%d: recvfrom: unexpected failure: errno=%d: %s\n",
+            sd, errno, strerror(errno));
 
     assert(0);
 
@@ -230,14 +256,14 @@ static void *on_udp_read_segment(oop_source *src, int sd,
 
   if (nepim_udp_hdr_parse(&hdr, buf, rd)) {
     fprintf(stderr, 
-	    "%d: recvfrom: could not parse header from UDP segment: %d/%d bytes\n",
-	    sd, rd, nepim_global.udp_read_size);
+            "%d: recvfrom: could not parse header from UDP segment: %d/%d bytes\n",
+            sd, rd, nepim_global.udp_read_size);
     return OOP_CONTINUE;
   }
 
   if (hdr.version) {
     fprintf(stderr, "%d: recvfrom: ignoring unknown UDP header version: %d\n",
-	    sd, hdr.version);
+            sd, hdr.version);
     return OOP_CONTINUE;
   }
 
@@ -246,29 +272,29 @@ static void *on_udp_read_segment(oop_source *src, int sd,
 
     if (!slot) {
       fprintf(stderr, "%d: recvfrom: unknown local slot: %d\n",
-	      sd, hdr.dst_slot);
+              sd, hdr.dst_slot);
       return OOP_CONTINUE;
     }
 
 #if 0
     fprintf(stderr, 
-	    "DEBUG FIXME %s %s: %d %d-%d in_seq=%d\n",
-	    __FILE__, __PRETTY_FUNCTION__,
-	    slot->udp_sd, hdr.dst_slot, hdr.src_slot, hdr.seq);
+            "DEBUG FIXME %s %s: %d %d-%d in_seq=%d\n",
+            __FILE__, __PRETTY_FUNCTION__,
+            slot->udp_sd, hdr.dst_slot, hdr.src_slot, hdr.seq);
 #endif
 
     if (slot->session.remote_len != fromlen) {
       fprintf(stderr, 
-	      "%d %d-%d: recvfrom: bad address length: expected=%d got=%d\n",
-	      sd, hdr.dst_slot, hdr.src_slot, 
-	      slot->session.remote_len, fromlen);
+              "%d %d-%d: recvfrom: bad address length: expected=%d got=%d\n",
+              sd, hdr.dst_slot, hdr.src_slot, 
+              slot->session.remote_len, fromlen);
       return OOP_CONTINUE;
     }
 
     if (memcmp(&slot->session.remote, &from, fromlen)) {
       fprintf(stderr, 
-	      "%d %d-%d: recvfrom: address mismatch\n",
-	      sd, hdr.dst_slot, hdr.src_slot);
+              "%d %d-%d: recvfrom: address mismatch\n",
+              sd, hdr.dst_slot, hdr.src_slot);
       return OOP_CONTINUE;
     }
 
@@ -282,7 +308,7 @@ static void *on_udp_read_segment(oop_source *src, int sd,
 
       slot->client_writer_status = NEPIM_SLOT_CLIENT_SEND;
     
-      udp_slot_start(slot);
+      schedule_udp_slot_start(slot);
     
       return OOP_CONTINUE;
     }
@@ -291,12 +317,14 @@ static void *on_udp_read_segment(oop_source *src, int sd,
 
     if (slot->index_remote != hdr.src_slot) {
       fprintf(stderr, 
-	      "%d %d-%d: recvfrom: remote index mismatch: expected=%d got=%d\n",
-	      sd, hdr.dst_slot, hdr.src_slot, slot->index_remote, hdr.src_slot);
+              "%d %d-%d: recvfrom: remote index mismatch: expected=%d got=%d\n",
+              sd, hdr.dst_slot, hdr.src_slot, slot->index_remote, hdr.src_slot);
       return OOP_CONTINUE;
     }
 
     udp_consume(slot, rd, hdr.seq);
+
+    udp_check_packet_data(sd, &hdr, &(slot->session), buf, UDP_HEADER_LEN, rd);
   }
 
   return OOP_CONTINUE;
@@ -323,6 +351,11 @@ static int slot_greet_write(nepim_slot_t *slot)
   opt.write_delay           = nepim_global.write_delay;
   opt.udp_keepalive_send    = nepim_global.udp_keepalive_require;
   opt.udp_keepalive_require = nepim_global.udp_keepalive_send;
+  opt.seed                  = nepim_global.seed;
+  opt.verify_data           = nepim_global.verify_data;
+  opt.random_fill           = nepim_global.random_fill;
+  opt.fill_byte             = nepim_global.fill_byte;
+  opt.pause_duration        = nepim_global.pause_duration;
 
   if (nepim_global.password) {
     opt.password_buf        = nepim_global.password;
@@ -352,14 +385,14 @@ static int slot_greet_write(nepim_slot_t *slot)
     case EAGAIN:
     case ECONNREFUSED:
       nepim_usock_write_error(&udp_tab, slot->udp_sd, 
-			      slot->index, slot->index_remote,
-			      errno);
+                              slot->index, slot->index_remote,
+                              errno);
       return 0; /* ignore as soft error (caller always resend) */
     default:
       fprintf(stderr, 
-	      "greet_write: error on UDP socket %d (%d-%d): %d: %s\n", 
-	      slot->udp_sd, slot->index, slot->index_remote, errno,
-	      strerror(errno));
+              "greet_write: error on UDP socket %d (%d-%d): %d: %s\n", 
+              slot->udp_sd, slot->index, slot->index_remote, errno,
+              strerror(errno));
     }
     assert(0); /* treat unexpected failures as fatal */
     return -3;
@@ -367,9 +400,9 @@ static int slot_greet_write(nepim_slot_t *slot)
 
 #if 0
   fprintf(stderr, 
-	  "%d %d-%d: sending: hdr_len=%d greet_len=%d total=%d wrote=%d\n", 
-	  slot->udp_sd, slot->index, slot->index_remote,
-	  UDP_HEADER_LEN, pr, write_len, wr);
+          "%d %d-%d: sending: hdr_len=%d greet_len=%d total=%d wrote=%d\n", 
+          slot->udp_sd, slot->index, slot->index_remote,
+          UDP_HEADER_LEN, pr, write_len, wr);
 #endif
 
   if (wr != write_len)
@@ -386,18 +419,18 @@ static void *on_udp_interval(oop_source *src, struct timeval tv, void *user)
   nepim_session_t *session = &slot->session;
 
   nepim_slot_stat(stdout, NEPIM_LABEL_PARTIAL, 
-		  slot->udp_sd,
-		  slot->index,
-		  slot->index_remote,
-		  session->byte_interval_recv,
-		  session->byte_interval_sent,
-		  session->stat_interval,
-		  session->tv_start.tv_sec,
-		  session->test_duration,
-		  session->interval_reads,
-		  session->interval_writes,
-		  slot->interval_pkt_lost,
-		  slot->interval_pkt_dup);
+                  slot->udp_sd,
+                  slot->index,
+                  slot->index_remote,
+                  session->byte_interval_recv,
+                  session->byte_interval_sent,
+                  session->stat_interval,
+                  session->tv_start.tv_sec,
+                  session->test_duration,
+                  session->interval_reads,
+                  session->interval_writes,
+                  slot->interval_pkt_lost,
+                  slot->interval_pkt_dup);
 
   session->byte_interval_recv = 0;
   session->byte_interval_sent = 0;
@@ -426,8 +459,8 @@ static void schedule_stat_interval(nepim_slot_t *slot)
   session->tv_interval.tv_sec += session->stat_interval;
 
   nepim_global.oop_src->on_time(nepim_global.oop_src,
-				session->tv_interval,
-				on_udp_interval, slot);
+                                session->tv_interval,
+                                on_udp_interval, slot);
 }
 
 static void *on_udp_duration(oop_source *src, struct timeval tv, void *user)
@@ -440,18 +473,18 @@ static void *on_udp_duration(oop_source *src, struct timeval tv, void *user)
   slot->total_pkt_dup += slot->interval_pkt_dup;
 
   nepim_slot_stat(stdout, NEPIM_LABEL_TOTAL, 
-		  sd, 
-		  slot->index,
-		  slot->index_remote,
-		  session->byte_total_recv,
-		  session->byte_total_sent,
-		  session->test_duration,
-		  session->tv_start.tv_sec,
-		  session->test_duration,
-		  session->total_reads,
-		  session->total_writes,
-		  slot->total_pkt_lost,
-		  slot->total_pkt_dup);
+                  sd, 
+                  slot->index,
+                  slot->index_remote,
+                  session->byte_total_recv,
+                  session->byte_total_sent,
+                  session->test_duration,
+                  session->tv_start.tv_sec,
+                  session->test_duration,
+                  session->total_reads,
+                  session->total_writes,
+                  slot->total_pkt_lost,
+                  slot->total_pkt_dup);
 
   session->byte_total_recv = 0;
   session->byte_total_sent = 0;
@@ -469,12 +502,12 @@ static void udp_slot_cancel_stat_timers(nepim_slot_t *slot)
   nepim_session_t *session = &slot->session;
 
   nepim_global.oop_src->cancel_time(nepim_global.oop_src,
-				    session->tv_duration,
-				    on_udp_duration, slot);
+                                    session->tv_duration,
+                                    on_udp_duration, slot);
 
   nepim_global.oop_src->cancel_time(nepim_global.oop_src,
-				    session->tv_interval,
-				    on_udp_interval, slot);
+                                    session->tv_interval,
+                                    on_udp_interval, slot);
 }
 
 static void cancel_slot_rate_write(nepim_slot_t *slot)
@@ -500,7 +533,7 @@ static void cancel_slot_rate_write(nepim_slot_t *slot)
     return;
 
   nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
-				  sd, OOP_WRITE);
+                                  sd, OOP_WRITE);
 }
 
 static void udp_write_greet(nepim_slot_t *slot)
@@ -524,15 +557,15 @@ static void udp_write_greet(nepim_slot_t *slot)
 
       slot->client_writer_status = NEPIM_SLOT_CLIENT_SEND;
     
-      udp_slot_start(slot);
+      schedule_udp_slot_start(slot);
 
       return;
     }
 
     fprintf(stderr, 
-	    "%d %d-%d: max. greetings reached (%d/%d) - server not responding\n",
-	    sd, slot->index, slot->index_remote,
-	    slot->greetings_sent, nepim_global.max_greetings);
+            "%d %d-%d: max. greetings reached (%d/%d) - server not responding\n",
+            sd, slot->index, slot->index_remote,
+            slot->greetings_sent, nepim_global.max_greetings);
 
     udp_slot_kill(slot);
 
@@ -540,13 +573,13 @@ static void udp_write_greet(nepim_slot_t *slot)
   }
 
   fprintf(stderr, "%d %d-%d: sending greetings\n",
-	  sd, slot->index, slot->index_remote);
+          sd, slot->index, slot->index_remote);
 
   result = slot_greet_write(slot);
   if (result) {
     fprintf(stderr, 
-	    "%d %d-%d: greet writing failed: %d\n", 
-	    sd, slot->index, slot->index_remote, result);
+            "%d %d-%d: greet writing failed: %d\n", 
+            sd, slot->index, slot->index_remote, result);
 
     udp_slot_kill(slot);
 
@@ -559,8 +592,8 @@ static void udp_write_greet(nepim_slot_t *slot)
 
   /* schedule for next time saved by on_udp_time_greet() */
   nepim_global.oop_src->on_time(nepim_global.oop_src,
-				session->tv_greet_rate,
-				on_udp_time_greet, slot);
+                                session->tv_greet_rate,
+                                on_udp_time_greet, slot);
 }
 
 static void udp_write_rate(nepim_slot_t *slot) 
@@ -602,8 +635,8 @@ static void udp_write_rate(nepim_slot_t *slot)
 
   if (session->rate_remaining > 0)
     to_write = NEPIM_RANGE(session->rate_remaining, 
-			   UDP_HEADER_LEN, 
-			   sizeof(buf));
+                           UDP_HEADER_LEN, 
+                           sizeof(buf));
   else
     to_write = sizeof(buf);
 
@@ -618,25 +651,25 @@ static void udp_write_rate(nepim_slot_t *slot)
     case EAGAIN:
     case ECONNREFUSED:
       nepim_usock_write_error(&udp_tab, sd, 
-			      slot->index, slot->index_remote,
-			      errno);
+                              slot->index, slot->index_remote,
+                              errno);
       return;
 
     case EPIPE:
       fprintf(stderr, 
-	      "rate_write: EPIPE on UDP socket %d (%d-%d)\n", 
-	      sd, slot->index, slot->index_remote);
+              "rate_write: EPIPE on UDP socket %d (%d-%d)\n", 
+              sd, slot->index, slot->index_remote);
       break;
     default:
       fprintf(stderr, 
-	      "rate_write: error on UDP socket %d (%d-%d): %d: %s\n", 
-	      sd, slot->index, slot->index_remote, errno,
-	      strerror(errno));
+              "rate_write: error on UDP socket %d (%d-%d): %d: %s\n", 
+              sd, slot->index, slot->index_remote, errno,
+              strerror(errno));
     }
 
     fprintf(stderr,
-	    "rate_write: connection lost on UDP socket %d (%d-%d)\n",
-	    sd, slot->index, slot->index_remote);
+            "rate_write: connection lost on UDP socket %d (%d-%d)\n",
+            sd, slot->index, slot->index_remote);
 
     if (!session->duration_done)
       report_broken_slot_stat(stdout, slot);
@@ -668,8 +701,8 @@ static void udp_write_rate(nepim_slot_t *slot)
 
     /* schedule for next time saved by on_udp_rate_delay() */
     nepim_global.oop_src->on_time(nepim_global.oop_src,
-				  session->tv_send_rate,
-				  on_udp_rate_delay, slot);
+                                  session->tv_send_rate,
+                                  on_udp_rate_delay, slot);
   }
 }
 
@@ -697,25 +730,25 @@ static void udp_write_full(nepim_slot_t *slot)
     case EAGAIN:
     case ECONNREFUSED:
       nepim_usock_write_error(&udp_tab, sd, 
-			      slot->index, slot->index_remote,
-			      errno);
+                              slot->index, slot->index_remote,
+                              errno);
       return;
 
     case EPIPE:
       fprintf(stderr, 
-	      "write: EPIPE on UDP socket %d (%d-%d)\n", 
-	      sd, slot->index, slot->index_remote);
+              "write: EPIPE on UDP socket %d (%d-%d)\n", 
+              sd, slot->index, slot->index_remote);
       break;
     default:
       fprintf(stderr, 
-	      "write: error on UDP socket %d (%d-%d): %d: %s\n", 
-	      sd, slot->index, slot->index_remote, errno,
-	      strerror(errno));
+              "write: error on UDP socket %d (%d-%d): %d: %s\n", 
+              sd, slot->index, slot->index_remote, errno,
+              strerror(errno));
     }
 
     fprintf(stderr,
-	    "write: connection lost on UDP socket %d (%d-%d)\n",
-	    sd, slot->index, slot->index_remote);
+            "write: connection lost on UDP socket %d (%d-%d)\n",
+            sd, slot->index, slot->index_remote);
 
     if (!session->duration_done)
       report_broken_slot_stat(stdout, slot);
@@ -734,7 +767,7 @@ static void udp_write_full(nepim_slot_t *slot)
 }
 
 static void *on_udp_write(oop_source *src, int sd,
-			  oop_event event, void *user)
+                          oop_event event, void *user)
 {
   char buf[nepim_global.udp_write_size];
   nepim_slot_t *slot;
@@ -759,7 +792,7 @@ static void *on_udp_write(oop_source *src, int sd,
   if (!session->must_send) {
     if (session->udp_keepalive_send) {
       nepim_udp_write_keepalive(slot, nepim_slot_buf_write,
-				on_udp_keepalive_time, udp_slot_kill);
+                                on_udp_keepalive_time, udp_slot_kill);
       return OOP_CONTINUE;
     }
   }
@@ -797,7 +830,7 @@ static void will_slot_rate_write(nepim_slot_t *slot)
     return;
 
   nepim_global.oop_src->on_fd(nepim_global.oop_src, sd,
-			      OOP_WRITE, on_udp_write, 0);
+                              OOP_WRITE, on_udp_write, 0);
 }
 
 static void *on_udp_rate_delay(oop_source *src, struct timeval tv, void *user)
@@ -826,9 +859,9 @@ static void *on_udp_rate_delay(oop_source *src, struct timeval tv, void *user)
 
   /* calculate bytes to be written from rate */
   session->rate_remaining = nepim_bps2bytes(session->max_bit_rate,
-					    session->write_delay);
+                                            session->write_delay);
   session->pkt_remaining = nepim_pps2packets(session->max_pkt_rate,
-					     session->write_delay);
+                                             session->write_delay);
 
 #ifndef NDEBUG
  {
@@ -862,7 +895,7 @@ static void cancel_slot_write(nepim_slot_t *slot)
     return;
 
   nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
-				  sd, OOP_WRITE);
+                                  sd, OOP_WRITE);
 }
 
 static void udp_slot_cancel_io(nepim_slot_t *slot)
@@ -880,14 +913,14 @@ static void udp_slot_cancel_io(nepim_slot_t *slot)
     if ((session->max_bit_rate > 0) || (session->max_pkt_rate > 0)) {
       /* stop current writing, if any */
       if (slot->want_write) {
-	assert(slot->want_write == 1);
-	cancel_slot_rate_write(slot);
+        assert(slot->want_write == 1);
+        cancel_slot_rate_write(slot);
       }
 
       /* stop periodic write scheduler */
       nepim_global.oop_src->cancel_time(nepim_global.oop_src,
-					session->tv_send_rate,
-					on_udp_rate_delay, slot);
+                                        session->tv_send_rate,
+                                        on_udp_rate_delay, slot);
     }
     else {
       cancel_slot_write(slot);
@@ -896,15 +929,15 @@ static void udp_slot_cancel_io(nepim_slot_t *slot)
   else {
     if (session->udp_keepalive_send) {
       if (slot->want_write) {
-	assert(slot->want_write == 1);
-	nepim_cancel_slot_keepalive(slot); /* keepalive write */
+        assert(slot->want_write == 1);
+        nepim_cancel_slot_keepalive(slot); /* keepalive write */
       }
       
       /* stop periodic keepalive scheduler */
       nepim_global.oop_src->cancel_time(nepim_global.oop_src,
-					session->tv_keepalive,
-					on_udp_keepalive_time,
-					slot);
+                                        session->tv_keepalive,
+                                        on_udp_keepalive_time,
+                                        slot);
     }
   }
 
@@ -920,8 +953,8 @@ static void udp_slot_kill(nepim_slot_t *slot)
 
   if (slot->session.udp_keepalive_require)
     nepim_global.oop_src->cancel_time(nepim_global.oop_src,
-				      slot->session.tv_keepalive_timer,
-				      on_udp_keepalive_require, slot);
+                                      slot->session.tv_keepalive_timer,
+                                      on_udp_keepalive_require, slot);
 
   cancel_slot_segment_read(slot);
 
@@ -939,7 +972,7 @@ static void will_slot_greet_writer(nepim_slot_t *slot)
     return;
 
   nepim_global.oop_src->on_fd(nepim_global.oop_src, sd,
-			      OOP_WRITE, on_udp_write, 0);
+                              OOP_WRITE, on_udp_write, 0);
 }
 
 static void cancel_slot_greet_writer(nepim_slot_t *slot)
@@ -953,7 +986,7 @@ static void cancel_slot_greet_writer(nepim_slot_t *slot)
     return;
 
   nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
-				  sd, OOP_WRITE);
+                                  sd, OOP_WRITE);
 }
 
 static void soft_cancel_slot_greet_writer(nepim_slot_t *slot)
@@ -967,8 +1000,8 @@ static void soft_cancel_slot_greet_writer(nepim_slot_t *slot)
 static void cancel_slot_greet_timer(nepim_slot_t *slot)
 {
   nepim_global.oop_src->cancel_time(nepim_global.oop_src,
-				    slot->session.tv_greet_rate,
-				    on_udp_time_greet, slot);
+                                    slot->session.tv_greet_rate,
+                                    on_udp_time_greet, slot);
 }
 
 static void will_slot_segment_read(nepim_slot_t *slot)
@@ -979,7 +1012,7 @@ static void will_slot_segment_read(nepim_slot_t *slot)
     return;
 
   nepim_global.oop_src->on_fd(nepim_global.oop_src, sd,
-			      OOP_READ, on_udp_read_segment, 0);
+                              OOP_READ, on_udp_read_segment, 0);
 }
 
 static void cancel_slot_segment_read(nepim_slot_t *slot)
@@ -990,11 +1023,11 @@ static void cancel_slot_segment_read(nepim_slot_t *slot)
     return;
 
   nepim_global.oop_src->cancel_fd(nepim_global.oop_src,
-				  sd, OOP_READ);
+                                  sd, OOP_READ);
 }
 
 static void *on_udp_time_greet(oop_source *src, struct timeval tv,
-			       void *user)
+                               void *user)
 {
   nepim_slot_t *slot = user;
   nepim_session_t *session;
@@ -1018,9 +1051,9 @@ static void *on_udp_time_greet(oop_source *src, struct timeval tv,
 }
 
 static int spawn_udp_connection(const char *hostname,
-				const char *portname,
-				struct sockaddr *remote, 
-				socklen_t *remote_len)
+                                const char *portname,
+                                struct sockaddr *remote, 
+                                socklen_t *remote_len)
 {
   struct addrinfo hints;
   struct addrinfo *ai_res;
@@ -1039,24 +1072,24 @@ static int spawn_udp_connection(const char *hostname,
   hints.ai_canonname = 0;
 
   fprintf(stderr, 
-	  "UDP socket solving %s,%s\n",
-	  hostname, portname);
+          "UDP socket solving %s,%s\n",
+          hostname, portname);
 
   result = getaddrinfo(hostname, portname,
-		       &hints, &ai_res);
+                       &hints, &ai_res);
   if (result) {
     fprintf(stderr, "%s: getaddrinfo(%s,%s): %s\n",
             __PRETTY_FUNCTION__, 
-	    hostname, portname,
-	    gai_strerror(result));
+            hostname, portname,
+            gai_strerror(result));
     return -1;
   }
 
   for (ai = ai_res; ai; ai = ai->ai_next) {
 
     fprintf(stderr, 
-	    "UDP socket trying %s,%s\n",
-	    hostname, portname);
+            "UDP socket trying %s,%s\n",
+            hostname, portname);
 
     if (nepim_global.no_inet6 && (ai->ai_family == PF_INET6))
       continue;
@@ -1065,24 +1098,24 @@ static int spawn_udp_connection(const char *hostname,
       continue;
 
     sd = nepim_connect_client_socket(ai->ai_addr, ai->ai_addrlen,
-				     ai->ai_family, ai->ai_socktype, 
-				     ai->ai_protocol,
-				     nepim_global.pmtu_mode,
-				     nepim_global.ttl,
-				     nepim_global.win_recv,
-				     nepim_global.win_send);
+                                     ai->ai_family, ai->ai_socktype, 
+                                     ai->ai_protocol,
+                                     nepim_global.pmtu_mode,
+                                     nepim_global.ttl,
+                                     nepim_global.win_recv,
+                                     nepim_global.win_send);
     if (sd < 0) {
       fprintf(stderr, 
-	      "could not connect UDP socket to %s,%s: %d\n",
-	      hostname, portname, sd);
+              "could not connect UDP socket to %s,%s: %d\n",
+              hostname, portname, sd);
       continue;
     }
 
     result = nepim_socket_mcast_ttl(sd, nepim_global.mcast_ttl);
     if (result) {
       fprintf(stderr,
-	      "%d: failure setting mcast_ttl=%d: %d\n",
-	      sd, nepim_global.mcast_ttl, result);
+              "%d: failure setting mcast_ttl=%d: %d\n",
+              sd, nepim_global.mcast_ttl, result);
     }
 
     /* save remote address */
@@ -1092,30 +1125,30 @@ static int spawn_udp_connection(const char *hostname,
 
     {
       union {
-	struct sockaddr_in inet;
-	struct sockaddr_in6 inet6;
+        struct sockaddr_in inet;
+        struct sockaddr_in6 inet6;
       } local_sockaddr;
       socklen_t local_sockaddr_len = sizeof(local_sockaddr);
       int result;
       char local[100];
       
       result = getsockname(sd, (struct sockaddr *) &local_sockaddr,
-			   &local_sockaddr_len);
+                           &local_sockaddr_len);
       assert(!result);
 
       nepim_sock_dump_addr(local, sizeof(local),
-			   (const struct sockaddr *) &local_sockaddr);
+                           (const struct sockaddr *) &local_sockaddr);
 
       fprintf(stderr, 
-	      "%d: UDP socket (%s,%d) connected to: "
-	      "host=%s,%s(%d) len=%d family=%d type=%d proto=%d\n",
-	      sd, 
-	      local,
-	      nepim_sock_get_port((struct sockaddr *) &local_sockaddr),
-	      hostname, portname, 
-	      nepim_sock_get_port(ai->ai_addr),
-	      ai->ai_addrlen, ai->ai_family, 
-	      ai->ai_socktype, ai->ai_protocol);
+              "%d: UDP socket (%s,%d) connected to: "
+              "host=%s,%s(%d) len=%d family=%d type=%d proto=%d\n",
+              sd, 
+              local,
+              nepim_sock_get_port((struct sockaddr *) &local_sockaddr),
+              hostname, portname, 
+              nepim_sock_get_port(ai->ai_addr),
+              ai->ai_addrlen, ai->ai_family, 
+              ai->ai_socktype, ai->ai_protocol);
     }
 
     break;
@@ -1139,8 +1172,8 @@ static void parse_hosts(const char *host_list)
     /* get hostname */
     if (addr_list_get(host_list, j, hostname, sizeof(hostname))) {
       fprintf(stderr, 
-	      "%s: failure parsing address %d/%d from list: %s\n",
-	      nepim_global.prog_name, j, size, host_list);
+              "%s: failure parsing address %d/%d from list: %s\n",
+              nepim_global.prog_name, j, size, host_list);
       continue;
     }
 
@@ -1151,24 +1184,24 @@ static void parse_hosts(const char *host_list)
     /* create sockets */
     for (i = 0; i < nepim_global.pipes; ++i) {
       union {
-	struct sockaddr_in inet;
-	struct sockaddr_in6 inet6;
+        struct sockaddr_in inet;
+        struct sockaddr_in6 inet6;
       } remote;
       int remote_len = sizeof(remote);
       int sd;
       nepim_greet_t opt;
 
       sd = spawn_udp_connection(hostname, portname, 
-				(struct sockaddr *) &remote, 
-				&remote_len);
+                                (struct sockaddr *) &remote, 
+                                &remote_len);
       if (sd < 0)
-	continue;
+        continue;
 
       /* add socket to table of readers/writers */
       nepim_usock_set_add(&udp_tab, sd);
 
       opt.must_send             = nepim_global.duplex_mode
-	|| nepim_global.simplex_client_send;
+        || nepim_global.simplex_client_send;
       opt.bit_rate              = nepim_global.bit_rate;
       opt.pkt_rate              = nepim_global.pkt_rate;
       opt.stat_interval         = nepim_global.stat_interval;
@@ -1176,16 +1209,21 @@ static void parse_hosts(const char *host_list)
       opt.write_delay           = nepim_global.write_delay;
       opt.udp_keepalive_send    = nepim_global.udp_keepalive_send;
       opt.udp_keepalive_require = nepim_global.udp_keepalive_require;
+      opt.seed                  = nepim_global.seed;
+      opt.verify_data           = nepim_global.verify_data;
+      opt.random_fill           = nepim_global.random_fill;
+      opt.fill_byte             = nepim_global.fill_byte;
+      opt.pause_duration        = nepim_global.pause_duration;
       
       /* create UDP slots */
       for (i = 0; i < nepim_global.pipes; ++i) {
-	int local_slot = nepim_slot_find_free(&slots);
-	int remote_slot = 0xFFFF;
+        int local_slot = nepim_slot_find_free(&slots);
+        int remote_slot = 0xFFFF;
 
-	nepim_slot_set_add(&slots, sd, 
-			   local_slot, remote_slot,
-			   (const struct sockaddr *) &remote,
-			   remote_len, &opt);
+        nepim_slot_set_add(&slots, sd, 
+                           local_slot, remote_slot,
+                           (const struct sockaddr *) &remote,
+                           remote_len, &opt);
       }
     }
 
@@ -1216,8 +1254,8 @@ void nepim_udp_clients(const char *host_list)
     */
     session->tv_greet_rate = OOP_TIME_NOW;
     nepim_global.oop_src->on_time(nepim_global.oop_src,
-				  session->tv_greet_rate,
-				  on_udp_time_greet, slot);
+                                  session->tv_greet_rate,
+                                  on_udp_time_greet, slot);
 
     /* wait answers */
     will_slot_segment_read(slot);
