@@ -18,7 +18,7 @@ Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
 MA 02110-1301 USA
 *-GNU-GPL-END-*/
 
-/* $Id: udp_server.c,v 1.69 2005/11/04 10:18:36 evertonm Exp $ */
+/* $Id: udp_server.c,v 1.71 2005/12/22 09:41:07 evertonm Exp $ */
 
 
 #include <stdio.h>
@@ -649,6 +649,44 @@ static void udp_slot_start(int local_slot)
     nepim_schedule_keepalive_timer(slot, on_udp_keepalive_require);
 }
 
+static void *on_udp_slot_start(oop_source *src, struct timeval tv, void *user)
+{
+  int local_slot = (int) user;
+
+  udp_slot_start(local_slot);
+
+  return OOP_CONTINUE;
+}
+
+static void schedule_udp_slot_start(int local_slot)
+{
+  nepim_slot_t *slot = nepim_slot_set_get(&slots, local_slot);
+  nepim_session_t *session;
+  struct timeval pause;
+
+  session = &slot->session;
+
+  if (session->pause_duration > 0) {
+    /* 
+       HACK:
+       Send back a packet as an ack,
+       otherwise the client might exit
+       with "server not responding"
+    */
+    char buf[nepim_global.udp_write_size];
+    (void) nepim_slot_buf_sendto(slot, buf, sizeof(buf), UDP_TYPE_DATA);
+  }
+
+  {
+      int result = gettimeofday(&pause, 0);
+      assert(!result);
+  }
+  pause.tv_sec += session->pause_duration;
+  nepim_global.oop_src->on_time(nepim_global.oop_src,
+				pause,
+				on_udp_slot_start, (void *) local_slot);
+}
+
 static int parse_greetings(nepim_greet_t *opt,
 			   int sd, int local_slot, int remote_slot,
 			   char *buf, const int buf_len,
@@ -662,16 +700,33 @@ static int parse_greetings(nepim_greet_t *opt,
   char password_buf[100];
 
   eos = memchr(buf, '\n', buf_len);
-  if (eos)
+  if (eos) {
     past_end = eos;
-  else
-    past_end = buf + buf_len;
+    *past_end = '\0';
+#if 0
+    fprintf(stderr, 
+	    "%d: %d-%d: %s: %s: minimum UDP read size (%d) needed for greeting.\n",
+	    sd, local_slot, remote_slot,
+	    __FILE__, __PRETTY_FUNCTION__,
+	    (UDP_HEADER_LEN + strlen(buf) + 1));
+#endif
+  } else {
+
+    /* 
+       Prevent greeting from being garbled when the UDP read size is
+       too small.
+    */
+    fprintf(stderr, 
+	    "%d: %d-%d: %s: %s: UDP read size (%d) too small for greeting.\n",
+	    sd, local_slot, remote_slot,
+	    __FILE__, __PRETTY_FUNCTION__,
+	    (UDP_HEADER_LEN + buf_len));
+    return 1;
+  }
 
   assert(past_end >= buf);
   assert(past_end <= (buf + buf_len));
 
-  *past_end = '\0';
-
   opt->password_buf = password_buf;
   opt->password_buf_size = sizeof(password_buf);
 
@@ -684,7 +739,8 @@ static int parse_greetings(nepim_greet_t *opt,
 
   fprintf(stderr, 
 	  "%d %d-%d: UDP from (%s,%d): send=%d bps=%lld pps=%d "
-	  "interval=%d duration=%d delay=%ld ka_send=%d ka_req=%d "
+	  "interval=%d duration=%d delay=%ld ka_send=%d ka_req=%d seed=0x%08x "
+	  "verify_data=%d random_fill=%d fill_byte=0x%02x pause_duration=%d "
 	  "password=%s\n", 
 	  sd, 
 	  local_slot, remote_slot,
@@ -693,7 +749,10 @@ static int parse_greetings(nepim_greet_t *opt,
 	  opt->must_send, opt->bit_rate, opt->pkt_rate,
 	  opt->stat_interval, opt->test_duration, 
 	  opt->write_delay, opt->udp_keepalive_send,
-	  opt->udp_keepalive_require, password_buf);
+	  opt->udp_keepalive_require, opt->seed,
+	  opt->verify_data, opt->random_fill, opt->fill_byte,
+	  opt->pause_duration,
+	  password_buf);
 
   if (nepim_global.tcpwrap) {
     if (!nepim_hosts_ctl(nepim_global.tcpwrap,
@@ -807,6 +866,22 @@ static void *on_udp_segment(oop_source *src, int sd,
       fprintf(stderr, 
 	      "%d %d-%d: normal behavior: ignoring existing remote src_slot/addr pair\n",
 	      sd, hdr.dst_slot, hdr.src_slot);
+
+      /* 
+	 HACK:
+	 Send back a packet as an ack,
+	 otherwise the client might exit
+	 with "server not responding"
+      */
+      {
+	nepim_slot_t *slot;
+	slot = nepim_slot_set_search_remote(&slots, 
+					    (const struct sockaddr *) &from,
+					    fromlen);
+	if (slot->session.pause_duration > 0)
+	  (void) nepim_slot_buf_sendto(slot, buf, sizeof(buf), UDP_TYPE_DATA);
+      }
+
       return OOP_CONTINUE;
     }
 
@@ -833,7 +908,7 @@ static void *on_udp_segment(oop_source *src, int sd,
 		       fromlen,
 		       &opt);
 
-    udp_slot_start(local_slot);
+    schedule_udp_slot_start(local_slot);
 
     return OOP_CONTINUE;
 
@@ -882,6 +957,8 @@ static void *on_udp_segment(oop_source *src, int sd,
     }
 
     udp_consume(slot, rd, hdr.seq);
+
+    udp_check_packet_data(sd, &hdr, &(slot->session), buf, UDP_HEADER_LEN, rd);
   }
 
   return OOP_CONTINUE;
@@ -965,6 +1042,3 @@ int nepim_udp_listener(const char *hostname, const char *portname,
 
   return udp_listeners;
 }
-
-
-
